@@ -1796,12 +1796,12 @@ do_create_process_fork(p_options *info, create_method method)
 
 // #pragma GCC diagnostic push
 // #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-//   if ( method == PCREATE_VFORK )
-//     pid = vfork();				/* As long as it is there we use it */
-//   else
+//  if ( method == PCREATE_VFORK )
+//    pid = vfork();				/* As long as it is there we use it */
+//  else
     pid = fork();
-  if ( pid != 0 )				/* parent */
-    restoreSignals(&set);
+//  if ( pid != 0 )				/* parent */
+//    restoreSignals(&set);
 // #pragma GCC diagnostic pop
 
   if ( pid == 0 )				/* child */
