@@ -332,6 +332,9 @@ CpuCount(void)
 void
 setOSPrologFlags(void)
 { PL_set_prolog_flag("cpu_count", PL_INTEGER, (intptr_t)CpuCount());
+#ifdef MSYS2
+  PL_set_prolog_flag("msys2", PL_BOOL|FF_READONLY, TRUE);
+#endif
 }
 
 
@@ -733,7 +736,7 @@ PRED_IMPL("win_add_dll_directory", 2, win_add_dll_directory, 0)
       /* AddDllDirectoryW() cannot handle "\\?\" */
       if ( (cookie = (*f_AddDllDirectoryW)(dirw + _xos_win_prefix_length(dirw))) )
       { DEBUG(MSG_WIN_API,
-	      Sdprintf("AddDllDirectory(%Ws) ok\n", dirw));
+	      SdprintfX("AddDllDirectory(%Ws) ok\n", dirw));
 
 	return PL_unify_int64(A2, (int64_t)(uintptr_t)cookie);
       }
@@ -807,7 +810,7 @@ PL_dlopen(const char *file, int flags)	/* file is in UTF-8, POSIX path */
     *w = 0;
   }
 
-  DEBUG(MSG_WIN_API, Sdprintf("dlopen(%Ws)\n", wfile));
+  DEBUG(MSG_WIN_API, SdprintfX("dlopen(%Ws)\n", wfile));
 
   if ( is_windows_abs_path(wfile) )
     llflags |= load_library_search_flags();
